Imports Microsoft.VisualBasic
Imports System
Imports System.IO
Imports System.IO.IsolatedStorage
Imports System.Collections.Generic


Namespace Samples
	Public Class DownloadCache
		''' <summary>
		''' Attempt to load the given XAP package from the cache.
		''' </summary>
		''' <param name="file">XAP file</param>
		''' <returns>Stream to the XAP package</returns>
		Public Shared Function Load(ByVal file As String) As Stream
			Dim iso As IsolatedStorageFile = IsolatedStorageFile.GetUserStoreForApplication()

			If (Not iso.FileExists(file)) Then
				iso.Dispose()
				Return Nothing
			End If

			' Check some expiration policy
			' File information is not available to Silverlight applications.
			' You might want to create a sort of index with entry date
			Dim m_Index As New CacheIndex()
			If (Not m_Index.IsExpired(file)) Then
				Return iso.OpenFile(file, FileMode.Open)
			End If

			' No load date available: reload
			iso.Dispose()
			Return Nothing
		End Function

		''' <summary>
		''' Write a XAP file with the content of the specified stream.
		''' </summary>
		''' <param name="xapName">Name of the package (file+ext)</param>
		''' <param name="xapStream">Stream with the content to save</param>
		Public Shared Sub Add(ByVal xapName As String, ByVal xapStream As Stream)
			Dim iso As IsolatedStorageFile = IsolatedStorageFile.GetUserStoreForApplication()
			If iso.FileExists(xapName) Then
				iso.DeleteFile(xapName)
			End If
			Dim fs As New IsolatedStorageFileStream(xapName, FileMode.CreateNew, iso)

			Dim buffer(32767) As Byte
			Dim bytesRead As Integer
			bytesRead = xapStream.Read(buffer, 0, buffer.Length)
			Do While bytesRead <> 0
				fs.Write(buffer, 0, bytesRead)
				bytesRead = xapStream.Read(buffer, 0, buffer.Length)
			Loop
			fs.Close()
			iso.Dispose()

			' Update index (overwrite if already exists)
			Dim m_Index As New CacheIndex()
			m_Index.Add(xapName)

		End Sub
	End Class
End Namespace
