Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Windows.Controls
Imports System.Net
Imports System.Reflection
Imports System.IO
Imports System.Windows
Imports System.Windows.Resources
Imports System.Windows.Markup



Namespace Samples
	Public Class DownloaderData
		Private privateXapName As String
		Public Property XapName() As String
			Get
				Return privateXapName
			End Get
			Set(ByVal value As String)
				privateXapName = value
			End Set
		End Property
		Private privateAssemblyName As String
		Public Property AssemblyName() As String
			Get
				Return privateAssemblyName
			End Get
			Set(ByVal value As String)
				privateAssemblyName = value
			End Set
		End Property
		Private privateClassName As String
		Public Property ClassName() As String
			Get
				Return privateClassName
			End Get
			Set(ByVal value As String)
				privateClassName = value
			End Set
		End Property
		Private privateActionRequired As PackageContent
		Public Property ActionRequired() As PackageContent
			Get
				Return privateActionRequired
			End Get
			Set(ByVal value As PackageContent)
				privateActionRequired = value
			End Set
		End Property
	End Class


	Public Enum PackageContent
		ClassFromAssembly = 1
		ResourceFromAssembly = 2
	End Enum

	Public Class Downloader
		Implements IDownloader
		#Region "Local members"
		Protected m_data As DownloaderData
		#End Region

		#Region "IDownloader Members"

		Public Event XapDownloaded As EventHandler(Of XapEventArgs) Implements IDownloader.XapDownloaded


		Public Sub LoadAssemblyFromPackage(ByVal xap As String, ByVal asm As String, ByVal cls As String) Implements IDownloader.LoadAssemblyFromPackage
			Initialize(xap, asm, cls, PackageContent.ClassFromAssembly)

			Dim bits As Stream = LookupCacheForPackage()
			If bits Is Nothing Then
				StartDownload()
			Else
				' Process and extract resources
				FindClassFromAssembly(bits)
			End If
		End Sub

		#End Region

		#Region "Helpers"

		Private Function LookupCacheForPackage() As Stream
			' Look up the XAP package for the assembly 
			Dim xapFile As String = m_data.XapName

			Return DownloadCache.Load(xapFile)
		End Function

		Private Sub StartDownload()
			Dim address As New Uri(m_data.XapName, UriKind.RelativeOrAbsolute)
			Dim client As New WebClient()

			Select Case m_data.ActionRequired
				Case PackageContent.ClassFromAssembly
					AddHandler client.OpenReadCompleted, AddressOf OnCompletionGetClassFromAssembly
				'case PackageContent.ResourceFromAssembly:
				'    client.OpenReadCompleted += new OpenReadCompletedEventHandler(OnCompletionGetResourceFromAssembly);
				'    break;

				Case Else
					Return
			End Select
			client.OpenReadAsync(address)
		End Sub

		Protected Sub Initialize(ByVal xap As String, ByVal asm As String, ByVal cls As String, ByVal action As PackageContent)
			If m_data Is Nothing Then
				m_data = New DownloaderData()
			End If

			m_data.AssemblyName = asm
			m_data.ClassName = cls
			m_data.XapName = xap
			m_data.ActionRequired = action
		End Sub

		#End Region


		#Region "Handlers"

		Private Sub OnCompletionGetClassFromAssembly(ByVal sender As Object, ByVal e As OpenReadCompletedEventArgs)
			' If no handler registered at the application level, why even bothering?
			If XapDownloadedEvent Is Nothing Then
				Return
			End If

			If e.Error IsNot Nothing Then
				Return
			End If

			' Save to the cache
			DownloadCache.Add(m_data.XapName, e.Result)

			' Process and extract resources
			FindClassFromAssembly(e.Result)
		End Sub

		Private Sub FindClassFromAssembly(ByVal content As Stream)
			' Load a particular assembly from XAP
			Dim a As System.Reflection.Assembly = GetAssemblyFromPackage(m_data.AssemblyName, content)

			' Get an instance of the specified user control class
			Dim page As Object = a.CreateInstance(m_data.ClassName)

			' Fire the event
			Dim args As New XapEventArgs()
			args.DownloadedContent = TryCast(page, UserControl)
			RaiseEvent XapDownloaded(Me, args)
		End Sub

		Private Function GetAssemblyFromPackage(ByVal assemblyName As String, ByVal xapStream As Stream) As System.Reflection.Assembly
			' Local variables
			Dim assemblyUri As Uri = Nothing
			Dim resPackage As StreamResourceInfo = Nothing
			Dim resAssembly As StreamResourceInfo = Nothing
			Dim part As AssemblyPart = Nothing

			' Initialize
			assemblyUri = New Uri(assemblyName, UriKind.Relative)
			resPackage = New StreamResourceInfo(xapStream, Nothing)
			resAssembly = Application.GetResourceStream(resPackage, assemblyUri)

			' Extract an assembly 
			part = New AssemblyPart()
			Dim a As System.Reflection.Assembly = part.Load(resAssembly.Stream)


			' Load other assemblies (dependencies) from manifest
			Dim manifestUri As New Uri("AppManifest.xaml", UriKind.Relative)
			Dim manifestStream As Stream = Application.GetResourceStream(resPackage, manifestUri).Stream
			Dim manifest As String = New StreamReader(manifestStream).ReadToEnd()

			' Parse the manifest using LINQ-to-XML to get the list of referenced assemblies
			Dim parts As List(Of AssemblyPart) = ManifestHelper.GetDeploymentParts(manifest)

			For Each ap As AssemblyPart In parts
				' Skip over primary assembly (done with it) and load all the others in memory
				If (Not ap.Source.ToLower().Equals(assemblyName)) Then
					Dim sri As StreamResourceInfo = Nothing
					sri = Application.GetResourceStream(resPackage, New Uri(ap.Source, UriKind.Relative))
					ap.Load(sri.Stream)
				End If
			Next ap

			' Close stream and returns
			xapStream.Close()
			Return a
		End Function

		#End Region    
	End Class
End Namespace
